"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.is_snowflake_uid = exports.create_snowflake_uid_generator = exports.validate_snowflake_conf = void 0;
const crypto_1 = require("crypto");
function validate_snowflake_conf(conf) {
    // todo: validate config
}
exports.validate_snowflake_conf = validate_snowflake_conf;
/**
 * Generates a unique 64-bit integer ID.
 *
 * Format based on Snowflake IDs (https://en.wikipedia.org/wiki/Snowflake_ID),
 * with the following modifications / details:
 *
 * - Uses a configurable epoch time
 * - Uses a configurable instance size (rather than the hard-coded 10-bits), and increases
 *   the size of the other fields if set to values smaller than 10.
 */
function create_snowflake_uid_generator(conf) {
    switch (conf.instance_size) {
        case 4n: return generator(15n, 0x7fffn, 0xfffffffffffn);
        case 5n: return generator(14n, 0x3fffn, 0xfffffffffffn);
        case 6n: return generator(14n, 0x3fffn, 0x7ffffffffffn);
        case 7n: return generator(13n, 0x1fffn, 0x7ffffffffffn);
        case 8n: return generator(13n, 0x1fffn, 0x3ffffffffffn);
        case 9n: return generator(12n, 0x0fffn, 0x3ffffffffffn);
        case 10n: return generator(12n, 0x0fffn, 0x1ffffffffffn);
    }
    function generator(sequence_size, sequence_mask, timestamp_mask) {
        const instance = BigInt(conf.instance) << sequence_size;
        const timestamp_offset = conf.instance_size + sequence_size;
        let sequence = BigInt((0, crypto_1.randomInt)(Number(sequence_mask)));
        function next_sequence() {
            const value = sequence++;
            sequence &= sequence_mask;
            return value;
        }
        return {
            uid() {
                const sequence = next_sequence();
                const timestamp = (now_big() - conf.epoch) & timestamp_mask;
                return BigInt.asUintN(64, (timestamp << timestamp_offset) | instance | sequence);
            },
            uid_str() {
                return this.uid().toString(10);
            }
        };
    }
}
exports.create_snowflake_uid_generator = create_snowflake_uid_generator;
function is_snowflake_uid(value) {
    return /^\d+$/.test(value) && value.length <= 20;
}
exports.is_snowflake_uid = is_snowflake_uid;
function now_big() {
    return BigInt(Date.now());
}
//# sourceMappingURL=index.js.map